""" goulash.python

    various shortcuts for python utilities
"""
import os
import glob

opj        = os.path.join
ope        = os.path.exists
ops        = os.path.split
opd        = os.path.dirname
listdir    = os.listdir
isdir      = os.path.isdir
makedir    = mkdir = os.mkdir
dirname    = os.path.dirname
splitext   = os.path.splitext
abspath    = os.path.abspath
getcwd     = os.getcwd
glob       = glob.glob
get_env    = os.environ.get
expanduser = os.path.expanduser
splitext   = os.path.splitext

def create_dir_if_not_exists(apath):
    """ returns a bool for 'created'
    """
    if not ope(apath):
        os.mkdir(apath)
        return True
    return False

def only_py_files(dir, rel=False):
    """ """
    result = glob(opj(dir, '*.py'))
    if rel: result = [ ops(fname)[-1] for fname in result]
    return result



""" goulash.classproperty

    SOURCE:
      http://stackoverflow.com/questions/128573/using-property-on-classmethods
"""
class classproperty(property):
    """
        USAGE:
          class constants:
              @classproperty
              def lazy(kls):
                  return "whatevs"
    """
    def __init__(self, func):
        return super(classproperty, self).__init__(classmethod(func))

    def __get__(self, obj, type_):
        return self.fget.__get__(None, type_)()

    def __set__(self, obj, value):
        cls = type(obj)
        return self.fset.__get__(None, cls)(value)



""" goulash.ansi
"""

from ansi2html.style import get_styles
from ansi2html import Ansi2HTMLConverter as _Ansi2HTMLConverter

class Ansi2HTMLConverter(_Ansi2HTMLConverter):
    """ Subclassed from ansi2html.Ans2HTMLConverter to prevent
        every convert() call redundantly rendering the style
    """
    def get_style(self):
        return '<style>' + \
               "\n".join(map(str, get_styles(self.dark_bg, self.scheme))) + \
               '</style>'

    def convert(self, *args, **kargs):
        kargs.update(full=False)
        return super(Ansi2HTMLConverter, self).convert(*args, **kargs)
Ansi2HtmlConverter=Ansi2HTMLConverter



""" goulash.wrappers
"""

class JSONWrapper(object):
    # convenience wrapper that makes __getattr__
    # work via __getitem__
    def __init__(self, data):
        self._data = data

    def __getattr__(self, name):
        try:
            return self._data[name]
        except KeyError:
            return getattr(self._data, name)
    __getitem__ = __getattr__

class DumbWrapper(object):
    # Simplest wrapper pattern
    def __init__(self, wrapped_obj):
        self._wrapped = wrapped_obj

    def __getattr__(self, name):
        if hasattr(self, name):
            print 'self acc'
            return object.__getattribute__(self, name)
        else:
            tmp = object.__getattribute__(self, '_wrapped')
            print 'wrap acc', name, tmp
            return getattr(tmp,name)

    def __iter__(self):
        return iter(self._wrapped)


class Dictionaryish(DumbWrapper):
    def __contains__(self, other):
        """ dictionary compatability """
        return other in self._wrapped

    def __getitem__(self,k):
        """ dictionary compatability """
        return self._wrapped[k]

    def keys(self):
        return self._wrapped.keys()

    def update(self, *args, **kargs):
        return self._wrapped.update(*args, **kargs)

    def __setitem__(self, *args, **kargs):
        return self._wrapped.__setitem__(*args, **kargs)



""" goulash.stdout

    (used by corkscrew.comet)
"""

import sys
import threading
from Queue import Queue, Empty

class ThreadedStdout(object): # pragma: no-cover
    """ Replacement for sys.stdout where every thread gets it's own
        queue.  Useful for capturing printed output from functions
        that are run in threads.  This is a toy, and not really a
        great idea to use except in short-lived programs.  See usage
        in corkscrew.comet
    """
    def __init__(self, stdout=None):
        self.stdout = stdout if stdout is not None else sys.stdout
        self.registry = {}

    def install(self):
        if sys.stdout != self:
            sys.stdout = self

    def __getattr__(self, x):
        return getattr(self.stdout, x)

    def register(self, thread):
        q = Queue()
        self.registry[thread] = q
        return q

    def read_all(self, thread):
        result = ""
        q = self.registry[thread]
        while q.qsize():
            try:
                result += q.get(block=False)
            except Empty:
                pass
        return result

    def write(self, data):
        this = threading.current_thread()
        if this in self.registry:
            self.registry[this].put(data)
        else:
            self.stdout.write(data)



""" goulash.cache

    dumb lightweight caching decorator.  this requires werkzeug,
    but at least avoids a memcache dependency.
"""

from functools import wraps


def cached(key_or_fxn, timeout=5 * 60, use_request_vars=False):
    """ dumb hack adapted from
        http://flask.pocoo.org/docs/patterns/viewdecorators/ """
    from werkzeug.contrib.cache import SimpleCache
    from goulash import cache as c
    if not getattr(c, 'CACHE', None):
        c.CACHE = SimpleCache()
    cache=c.CACHE
    if use_request_vars:
        tmp1 = key_or_fxn
        assert isinstance(key_or_fxn, basestring)
        def cache_key_fxn():
            from flask import request
            req = request
            z = sorted(req.values.items())
            import json
            key = tmp1 + json.dumps(z)
            return key
    elif isinstance(key_or_fxn, basestring):
        cache_key_fxn = lambda: key_or_fxn
    def decorator(f):
        @wraps(f)
        def decorated_function(*args, **kwargs):
            tmp2 = cache_key_fxn()
            rv = cache.get(tmp2)
            if rv is not None:
                return rv
            rv = f(*args, **kwargs)
            cache.set(tmp2, rv, timeout=timeout)
            return rv
        return decorated_function
    return decorator



""" goulash.metaclasses

    random experiments with metaclasses.

    you probably don't want to use this stuff :)
"""
import new, copy

from collections import defaultdict

from goulash.util import uniq

subclass_registry = defaultdict(lambda:[])

def metaclass_hook(func):
    func.metaclass_hook = True
    return staticmethod(func)

def dynamic_name(): return 'DynMix({U})'.format(U=uniq())

class META(type):
    """ the most generic metaclass.

        all this does is provide support for hooks.  by default the
        class-registration-hook is turned on, but if you want that
        turned off just subclass this and set `metaclass_hooks=[]`

        NB: to avoid MRO issues, this should be the main metaclass
            that's used, and everything else should subclass it.

        metaclass_registration_hook::
          this tracks class-hierarchy information for every class
          that uses this metaclass.  it keeps a dictionary of this
          form updated:

             subclass_registry[<__bases__ list>] = [<subclass1>, ..]
    """

    @staticmethod
    def enumerate_metaclass_hooks(mcls):
        """ returns a dictionary of metaclass hooks
            that will be run along with __new___
        """
        # TODO: use Namespace()
        matches = [ x for x in dir(mcls) if \
                    getattr(getattr(mcls, x, None),
                            'metaclass_hook', False) ]
        return dict( [ [match, getattr(mcls, match)] for match in matches ] )

    @metaclass_hook
    def metaclass_class_registration_hook(mcls, name, bases, dct, class_obj):
        """ called when initializing (configuring) class,
            this method records data about hierarchy structure
        """
        subclass_registry[bases].append(class_obj)

    def __new__(mcls, name, bases, dct):
        """ simply reproduce the usual behaviour of type.__new__
            run any hooks (hooks are defined by subclassers)
        """
        try:
            class_obj = type.__new__(mcls, name, bases, dct)
        except TypeError,e:
            # probably the cannot create consistent MRO error
            print dict( [ [b.__name__,
                           getattr(b,'__metaclass__',None)] for b in bases])
            raise e
        hooks = mcls.metaclass_hooks if hasattr(mcls, 'metaclass_hooks') else \
                mcls.enumerate_metaclass_hooks(mcls)
        for hook in hooks.values():
            hook(mcls, name, bases, dct, class_obj)
        return class_obj


class ClassAlgebra(META):
    """ a metaclass that tracks it's subclasses. """

    def __lshift__(kls, my_mixin):
        """ algebra for left-mixin

             The following are equivalent:
              >>>  my_class = my_class << my_mixin
              >>>  class my_class(my_mixin, my_class): pass
        """
        name  = dynamic_name()
        bases = (my_mixin, kls)
        return kls.__metaclass__(name, bases, {})

    def __rshift__(kls, my_mixin):
        """ algebra for right-mixin:

             The following are equivalent:
              >>> my_class = my_class >> my_mixin
              >>> class my_class(my_class,my_mixin): pass
        """
        name  = dynamic_name()
        bases = (kls, my_mixin)
        return kls.__metaclass__(name, bases, {})

    def subclass(kls, name=None, dct={}, **kargs):
        """ dynamically generate a subclass of this class """
        dct = copy.copy(dct)
        dct.update(kargs)
        if hasattr(kls, '_subclass_hooks'):
            # TODO: shouldnt be here, abstract this
            name, dct = kls._subclass_hooks(name=name, **dct)
        name = name or "DynamicSubclassOf{K}_{U}".format(K=kls.__name__,
                                         U=uniq())
        # why does this behave differently than type() ?
        return new.classobj(name, (kls,), dct)

META1 = ClassAlgebra

def supports_class_algebra(kls):
    """ for use as a decorator
    """
    if hasattr(kls,'__metaclass__'):
        if kls.__metaclass__!=ClassAlgebra:
            raise TypeError("{0} already has a metaclass: '{1}'".
                            format(kls,kls.__metaclass__))
        else:
            return kls
    else:
        class Temp(kls):
            __metaclass__  = ClassAlgebra
        Temp.__name__ = kls.__name__
        return Temp

def subclass_tracker(*bases, **kargs):
    """ dynamically generates the subclass tracking class that extends ``bases``.

        often the name doesn't matter and will never be seen,
        but you might as well be verbose in case it's stumbled across.

        usually an empty dictionary is fine for the namespace.. after all you're
        specifying the bases already, right?

        Example usage follows:

          SomeService(classtracker(Service, Mixin1, Mixin2)):
               ''' function body '''

    """
    if kargs:
        assert kargs.keys() == ['namespace'],'only the namespace kw arg is defined'
        namespace = kargs.pop('namespace')
    else:
        namespace = {}
    name = 'DynamicallyGeneratedClassTracker'
    return META(name, bases, namespace)



""" goulash.util
"""
import os
import time, uuid

def summarize_fpath(fpath):
    """ truncates a filepath to be more suitable for display.
        every instance of $HOME is replaced with ~
    """
    home = os.environ.get('HOME', None)
    if home:
        return fpath.replace(home, '~')

def uniq(use_time=False):
    """ """
    result = str(uuid.uuid1())
    if use_time: result+=str(time.time())[:-3]
    return result



""" goulash.parsing
"""
import re
from HTMLParser import HTMLParser

R_SPLIT_DELIM = re.compile('[\W_]+')

class MLStripper(HTMLParser):
    # utility to strip html which only requires the pyton stdlib.  taken from:
    #  http://stackoverflow.com/questions/753052/strip-html-from-strings-in-python
    def __init__(self):
        self.reset()
        self.fed = []

    def handle_data(self, d):
        self.fed.append(d)

    def get_data(self):
        return ''.join(self.fed)

def strip_tags(html):
    s = MLStripper()
    s.feed(html)
    return s.get_data()

def smart_split(x):
    """ splits on most delims """
    return [x for x in R_SPLIT_DELIM.split(x) if x]

def sanitize_txt(x):
    """ make text suitable for href linking, etc """
    return '_'.join(smart_split(x.lower()))



""" goulash._os
"""
import os

from goulash.python import get_env, ope

def home():
    return get_env('HOME')
get_home = home

def touch_file(_file):
    """ create _file if it does not exist  """
    if not ope(_file):
        with open(_file, 'w'):
            pass

def which(name):
    return os.popen('which '+name).readlines()[0].strip()

def get_mounts_by_type(mtype):
    tmp = os.popen('mount -l -t {0}'.format(mtype))
    tmp = tmp.readlines()
    tmp = [x.strip() for x in tmp if x.strip()]
    tmp2 = []
    for line in tmp:
        mdata = dict(line=line)
        line = line.split(' on ')
        name = line.pop(0)
        line = ''.join(line)
        line = line.split(' type ')
        mount_point = line.pop(0)
        mdata.update(name=name, mount_point=mount_point)
        tmp2.append(mdata)
    return tmp2



""" goulash.settings

    Standard configuration parser suitable to inherit
    for all kinds of applications.  This parses ini files
    and command line options, then combines the information.
    As a result of having all the information it is convenient
    (if not exactly correct) to treat the settings object
    as an "entry point".
"""
import os
import configparser

class SettingsError(Exception):
    pass

class GoulashConfigParser(configparser.ConfigParser):
    pass

class Settings(object):

    default_file = None
    environ_key = None
    #__getitem__ = Dictionaryish.__getitem__

    def get_parser(self):
        """ build the default parser """
        from optparse import OptionParser
        parser = OptionParser()
        parser.set_conflict_handler("resolve")
        parser.add_option(
            "-c", default='',dest='cmd',
            help=("just like python -c or sh -c (pass in a command)"))
        parser.add_option(
            "-e", "--exec", default='',dest='execfile',
            help='a filename to execute')
        parser.add_option(
            "-v", '--version', default=False, dest='version',
            action='store_true',
            help=("show version information"))
        parser.add_option("--shell", dest="shell",
                          default=False, help="application shell",
                          action='store_true')
        parser.add_option("--config", dest='config',
                          default="",
                          help="use config file")
        return parser

    def get_section(self, k, insist=False):
        try:
            return self[k]
        except KeyError:
            if insist:
                error = 'Fatal: You need to specify a "{0}" section in {1}'
                raise SettingsError(error.format(k, self.settings_file))
            else:
                return None

    def get_setting(self, k, insist=False, default=None):
        """ TODO: move into goulash
            this function returns True for 'true', False for
            `false` or 'no', any other strings are passed through
        """
        _type = 'val'
        section,var = k.split('.')
        section_obj = self.get_section(section, insist=insist)
        try:
            tmp = section_obj[var]
        except KeyError:
            if insist:
                error = ('Fatal: You need to specify a "{0}" section '
                             'with an entry like  "{1}=<{2}>" in {3}')
                raise SettingsError(error.format(
                    section, var, _type, self.settings_file))
            elif default:
                tmp = default
            else:
                return None
        if isinstance(tmp, basestring):
            test = tmp not in ['0', 'false', 'no', 0]
            if not test: return test
        return tmp

    @property
    def settings_file(self):
        if self.options is not None and self.options.config:
            _file = self.options.config
        else:
            _file = self._init_filename or \
                    os.environ.get(self.environ_key) or \
                    self.default_file
        _file = os.path.expanduser(_file)
        return _file

    def pre_run(self):
        """ hook for subclassers.. """
        pass

    def load(self, file, config={}):
        """ returns a dictionary with key's of the form
            <section>.<option> and the values
        """
        if not os.path.exists(file):
            err = 'ERROR: config file at "{f}" does not exist'.format(f=file)
            raise SettingsError(err)
        config = config.copy()
        cp = GoulashConfigParser()
        cp.read(file)
        return cp

    def shell_namespace(self):
        """ when --shell is used, the dictionary
            returned by this function populates
            all the shell variables """
        return {}

    def show_version(self):
        """ subclassers should call super
            and then print their own junk """
        from goulash import __version__
        print 'goulash=={0}'.format(__version__)

    def run(self, *args, **kargs):
        """ this is a thing that probably does not belong in a
            settings abstraction, but it is very useful..
        """
        self.pre_run()

        if self.options.cmd:
            ns = globals()
            ns.update(settings=self)
            exec self.options.cmd in self.shell_namespace()
            return True

        if self.options.execfile:
            ns = globals()
            ns.update(settings=self)
            execfile(self.options.execfile, self.shell_namespace())
            return True

        if self.options.version:
            return self.show_version()

        if self.get_setting('user.shell'):
            try:
                from smashlib import embed;
            except ImportError:
                raise SettingsError("You need smashlib installed "
                                    "if you want to use the shell.")
            else:
                embed(user_ns = self.shell_namespace())
            return True

    def __init__(self, filename=None, use_argv=True):
        """ first load the default config so that overrides don't need
            to mention everything.  update default with the config
            specified by the command line optionparser, then
            update that with any other overrides delivered to the parser.
        """
        self._init_filename = filename
        #super(Settings, self).__init__({})
        if use_argv:
            self.options, self.args = self.get_parser().parse_args()
        else:
            self.options = self.args = None
        self._wrapped = self.load(file=self.settings_file)
        # build a special dynamic section for things the user wants,
        # ie, things that have been passed into the option
        # parser but are not useful in the .ini
        if not self.get_section('user'):
            self['user'] = {}
        if self.options is not None:
            self['user']['shell'] = self.options.shell and 'true' or ''
        else:
            self['user']['shell'] = ''

    def __contains__(self, other):
        """ dictionary compatability """
        return other in self._wrapped

    def __getitem__(self,k):
        """ dictionary compatability """
        return self._wrapped[k]

    def keys(self):
        return self._wrapped.keys()

    def __setitem__(self, *args, **kargs):
        return self._wrapped.__setitem__(*args, **kargs)



""" goulash.net

    This is basically a bunch of neat stuff you can do with
    the socket library, and if I need it I dont want to look
    it up again.  I've also only tested it in linux, so YMMV
"""

import socket

def is_port_open(port, ip):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        s.connect((ip, int(port)))
        s.shutdown(2)
        return True
    except:
        return False

def ipaddr_with_LAN(host = "gmail.com"):
    """ you need to make an outbound connection to get
        an ip-address on a particular interface.. so this
        function requires interwebs.

        this might actually be an address on the internet,
        more likely you are behind a router and this is
        a class C.
    """
    port = 80
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect((host, port))
    result = s.getsockname()
    if len(result)==2:
        return result[0]

def ipaddr_hosts():
    """ on linux, uses /etc/hosts, returns something like this:

         ('cormac', ['localhost', 'testserver'], ['127.0.0.1'])
    """
    name, aliaslist, addresslist = socket.gethostbyname_ex(socket.gethostname())
    return name, aliaslist, addresslist

def ipaddr_basic():
    """ two different approaches just for reference purposes..
        these results are always the same for me, but ymmv depending
        on setup and platform..
    """
    x = []
    x.append(socket.gethostbyname(socket.gethostname()))
    x.append(socket.gethostbyname(socket.getfqdn()))
    return set(map(str, x))



#!/usr/bin/env python
""" setup.py for goulash
"""

import os, sys
from setuptools import setup

# make sure that finding packages works, even
# when setup.py is invoked from outside this dir
this_dir = os.path.dirname(os.path.abspath(__file__))
if not os.getcwd()==this_dir:
    os.chdir(this_dir)

# make sure we can import the version number so that it doesn't have
# to be changed in two places. goulash/__init__.py is also free
# to import various requirements that haven't been installed yet
sys.path.append(os.path.join(this_dir, 'goulash'))
from version import __version__
sys.path.pop()

base_url = 'https://github.com/mattvonrocketstein/goulash/'
setup(
    name         = 'goulash',
    version      = __version__,
    description  = 'toolbox, random shared stuff from my other projects',
    author       = 'mattvonrocketstein',
    author_email = '$author@gmail',
    url          = base_url,
    download_url = base_url+'/tarball/master',
    packages     = ['goulash'],
    keywords     = ['goulash'],
    install_requires = ['ansi2html', 'configparser', 'addict']
    )



""" goulash.venv
"""
import os
import fnmatch
from goulash.python import opj, ope, expanduser, abspath

get_path   = lambda: os.environ['PATH']
get_venv   = lambda: os.environ.get('VIRTUAL_ENV', None)
to_vbin    = lambda venv: opj(venv, 'bin')
to_vlib    = lambda venv: opj(venv, 'lib')
venv_bin   = lambda cmd: opj(to_vbin(get_venv()), cmd)

def is_venv(dir):
    """ naive.. but seems to work
        TODO: find a canonical version of this function or refine it
    """
    if ope( opj(dir, 'bin', 'activate_this.py')):
        return dir


def contains_venv(_dir, **kargs):
    """ ascertain whether _dir is, or if it contains, a venv.
        returns the first matching path according to the heuritic:

            1) if the directoy is a venv, return it
            2) if the directory has subdir(s) that are venvs, return the first
            3) no venv found?  return None
    """
    kargs.update(max_venvs=1)
    venvs = find_venvs(_dir, **kargs)
    return venvs and venvs[0]

def find_venvs(_dir, report=None, max_venvs=None, ignore_dirs=[]):
    _dir = abspath(expanduser(_dir))
    venvs = []
    if is_venv(_dir):
        return venvs.append(_dir)

    count = 1
    for dirpath, dirnames, filenames in os.walk(_dir):
        if len(venvs) == max_venvs:
            break
        # trick to make sure we dont process .git/.tox first, etc
        dirnames = [x for x in reversed(sorted(dirnames))]
        for subdir in dirnames:
            count += 1
            subdir = opj(dirpath, subdir)
            if is_venv(subdir):
                if not any([
                    fnmatch.fnmatch(
                        subdir, os.path.join('*',d,'*')) for d in ignore_dirs]):
                    venvs.append(subdir)

    if report is not None and not venvs:
        assert callable(report)
        msg = "contains_venv({0}):"
        report(msg.format(_dir))
        msg = "  searched {0} subdirectories: found no python venv's"
        msg = msg.format(count)
        report(msg)

    return venvs



""" goulash._inspect
"""
import inspect
from addict import Dict

def getcaller(level=2):
    """ """
    x = inspect.stack()[level]
    frame = x[0]
    file_name = x[1]
    flocals = frame.f_locals
    fglobals = frame.f_globals
    func_name = x[3]
    himself = flocals.get('self', None)
    try:
        kls  = himself and himself.__class__
    except AttributeError:
        # python uses self only by convention, so it's
        # possible there is a "himself" local but it's
        # not actually an object.
        kls = None
    kls_func = getattr(kls, func_name, None)
    if type(kls_func)==property:
        func = kls_func
    else:
        try:
            func = himself and getattr(himself, func_name)
        except AttributeError:
            func = func_name+'[nested]'
    out = dict(file=file_name,
               self=himself,
               locals=flocals,
               globals=fglobals,
               func=func,
               func_name=func_name)
    out.update({'class':kls})
    return Dict(out)
get_caller = getcaller
